/*
** pork_irc_output.c
** Copyright (C) 2003-2004 Ryan McCabe <ryan@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <errno.h>

#include <pork_util.h>
#include <pork_list.h>
#include <pork_queue.h>
#include <pork_inet.h>
#include <pork_io.h>
#include <pork_acct.h>
#include <pork_opt.h>
#include <pork_irc.h>

static int irc_send_server(int sock, char *cmd) {
	return (sock_write(sock, cmd, strlen(cmd)));
}

int irc_flush_outq(irc_session_t *session) {
	char *cmd;
	int ret = 0;

	while ((cmd = queue_get(session->outq)) != NULL) {
		ret++;
		irc_send_server(session->sock, cmd);
		free(cmd);
	}

	return (ret);
}

int irc_connect(struct pork_acct *acct,
				const char *server,
				char *port,
				int *sock)
{
	struct sockaddr_storage ss;
	struct sockaddr_storage local;
	in_port_t port_num;
	char *irchost = getenv("IRCHOST");

	if (server == NULL || port == NULL)
		return (-1);

	memset(&ss, 0, sizeof(ss));
	memset(&local, 0, sizeof(local));

	if (get_port(port, &port_num) != 0)
		return (-1);

	if (get_addr(server, &ss) != 0)
		return (-1);

	if (irchost != NULL) {
		if (get_addr(irchost, &local) != 0)
			memcpy(&local, &acct->laddr, sizeof(local));
	} else
		memcpy(&local, &acct->laddr, sizeof(local));

	sin_set_port(&local, acct->lport);
	return (nb_connect(&ss, &local, port_num, sock));
}

int irc_send(irc_session_t *session, char *command) {
	if (session->sock < 0) {
		char *cmd;

		cmd = xstrdup(command);
		if (queue_add(session->outq, cmd) != 0) {
			free(cmd);
			return (-1);
		}

		return (0);
	}

	return (irc_send_server(session->sock, command));
}

int irc_send_mode(irc_session_t *session, char *mode_str) {
	int ret;
	char buf[2048];

	ret = snprintf(buf, sizeof(buf), "MODE %s\r\n", mode_str);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_pong(irc_session_t *session, char *dest) {
	char buf[2048];
	int ret;

	ret = snprintf(buf, sizeof(buf), "PONG %s\r\n", dest);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_set_away(irc_session_t *session, char *msg) {
	char buf[2048];
	int ret;

	if (msg != NULL)
		ret = snprintf(buf, sizeof(buf), "AWAY :%s\r\n", msg);
	else
		ret = snprintf(buf, sizeof(buf), "AWAY\r\n");

	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_raw(irc_session_t *session, char *command) {
	return (irc_send(session, command));
}

int irc_send_join(irc_session_t *session, char *channel, char *key) {
	char buf[2048];
	int ret;

	if (key != NULL)
		ret = snprintf(buf, sizeof(buf), "JOIN %s %s\r\n", channel, key);
	else
		ret = snprintf(buf, sizeof(buf), "JOIN %s\r\n", channel);

	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_login(irc_session_t *session) {
	char buf[2048];
	struct pork_acct *acct = session->data;
	int ret;

	if (acct->passwd != NULL) {
		ret = snprintf(buf, sizeof(buf), "PASS %s\r\n", acct->passwd);
		if (ret < 0 || (size_t) ret >= sizeof(buf))
			return (-1);

		ret = irc_send(session, buf);
		if (ret == -1)
			return (-1);
	}

	ret = snprintf(buf, sizeof(buf), "NICK %s\r\n", acct->username);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	ret = irc_send(session, buf);
	if (ret == -1)
		return (-1);

	ret = snprintf(buf, sizeof(buf), "USER %s ojnk ojnk :%s\r\n",
			acct->username, acct->profile);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_privmsg(irc_session_t *session, char *dest, char *msg) {
	char buf[2048];
	int ret;

	ret = snprintf(buf, sizeof(buf), "PRIVMSG %s :%s\r\n", dest, msg);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_ctcp(irc_session_t *session, char *dest, char *msg) {
	char buf[2048];
	int ret;

	ret = snprintf(buf, sizeof(buf), "PRIVMSG %s :\x01%s\x01\r\n", dest, msg);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_names(irc_session_t *session, char *chan) {
	char buf[2048];
	int ret;

	if (chan != NULL)
		ret = snprintf(buf, sizeof(buf), "NAMES :%s\r\n", chan);
	else
		ret = snprintf(buf, sizeof(buf), "NAMES\r\n");

	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_who(irc_session_t *session, char *dest) {
	char buf[2048];
	int ret;

	if (dest != NULL)
		ret = snprintf(buf, sizeof(buf), "WHO :%s\r\n", dest);
	else
		ret = snprintf(buf, sizeof(buf), "WHO\r\n");

	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_whois(irc_session_t *session, char *dest) {
	char buf[2048];
	int ret;
	struct pork_acct *acct = session->data;

	if (dest == NULL)
		dest = acct->username;

	ret = snprintf(buf, sizeof(buf), "WHOIS %s\r\n", dest);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_whowas(irc_session_t *session, char *dest) {
	char buf[2048];
	int ret;
	struct pork_acct *acct = session->data;

	if (dest == NULL)
		dest = acct->username;

	ret = snprintf(buf, sizeof(buf), "WHOWAS %s\r\n", dest);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_nick(irc_session_t *session, char *nick) {
	char buf[2048];
	int ret;

	if (nick == NULL)
		return (-1);

	ret = snprintf(buf, sizeof(buf), "NICK :%s\r\n", nick);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_part(irc_session_t *session, char *chan) {
	char buf[2048];
	int ret;

	if (chan == NULL)
		return (-1);

	ret = snprintf(buf, sizeof(buf), "PART %s\r\n", chan);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_quit(irc_session_t *session, char *reason) {
	char buf[2048];
	int ret;

	if (reason == NULL)
		reason = DEFAULT_IRC_QUITMSG;

	ret = snprintf(buf, sizeof(buf), "QUIT :%s\r\n", reason);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	pork_io_del(session);
	return (irc_send(session, buf));
}

int irc_send_notice(irc_session_t *session, char *dest, char *msg) {
	char buf[2048];
	int ret;

	ret = snprintf(buf, sizeof(buf), "NOTICE %s :%s\r\n", dest, msg);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_kick(irc_session_t *session, char *chan, char *nick, char *msg) {
	char buf[2048];
	int ret;

	ret = snprintf(buf, sizeof(buf), "KICK %s %s :%s\r\n", chan, nick, msg);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_topic(irc_session_t *session, char *chan, char *topic) {
	char buf[2048];
	int ret;

	if (topic != NULL)
		ret = snprintf(buf, sizeof(buf), "TOPIC %s :%s\r\n", chan, topic);
	else
		ret = snprintf(buf, sizeof(buf), "TOPIC %s\r\n", chan);

	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}

int irc_send_action(irc_session_t *session, char *dest, char *msg) {
	char buf[4096];
	int ret;

	if (dest == NULL)
		return (-1);

	if (msg == NULL)
		msg = "";

	ret = snprintf(buf, sizeof(buf), "PRIVMSG %s :\x01" "ACTION %s\x01\r\n",
			dest, msg);
	if (ret < 0 || (size_t) ret >= sizeof(buf))
		return (-1);

	return (irc_send(session, buf));
}
